@@ -7,16 +7,20 @@ use self::Order::*;
 /// Set to `true` to see all the steps of β-reductions. The default is `false`.
 pub const SHOW_REDUCTIONS: bool = false;
 
-/// The [evaluation order](https://en.wikipedia.org/wiki/Lambda_calculus#Reduction_strategies) of 
-/// β-reductions. `Applicative` order will not fully reduce expressions containing functions 
+/// The [evaluation order](https://en.wikipedia.org/wiki/Lambda_calculus#Reduction_strategies) of
+/// β-reductions. `Applicative` variants will not fully reduce expressions containing functions
 /// without a normal form, e.g. the Y combinator. The default is `Normal`.
 pub const EVALUATION_ORDER: Order = Normal;
 
 /// The available variants of β-reduction evaluation order.
 #[derive(Debug, PartialEq)]
 pub enum Order {
+    /// leftmost outermost
     Normal,
-    Applicative
+    /// leftmost innermost
+    ApplicativeLeft,
+    /// rightmost innermost
+    ApplicativeRight
 }
 
 /// Applies two terms with substitution and variable update, consuming the first one in the process.
@@ -119,10 +123,22 @@ impl Term {
     /// assert_eq!(succ_one, succ().apply(&1.into()).unwrap());
     /// ```
     pub fn beta_once(&mut self) {
+        self.beta_once_indicative();
+    }
+
+    fn beta_once_indicative(&mut self) -> bool {
         match EVALUATION_ORDER {
             Normal => self._beta_once_normal(0),
-            Applicative => self._beta_once_applicative(0)
-        };
+            ApplicativeLeft => self._beta_once_applicative_l(0),
+            ApplicativeRight => self._beta_once_applicative_r(0)
+        }
+    }
+
+    fn reduce(&mut self, depth: u32) {
+        let copy = self.clone();
+        if SHOW_REDUCTIONS { print!("    {} reduces to ", show_precedence(self, 0, depth)) };
+        *self = copy.eval().unwrap();
+        if SHOW_REDUCTIONS { println!("{}", show_precedence(self, 0, depth)) }
     }
 
     // the return value indicates if reduction was performed
@@ -132,48 +148,56 @@ impl Term {
             Abs(_) => self.unabs_ref_mut().unwrap()._beta_once_normal(depth + 1),
             App(_, _) => {
                 if self.lhs_ref().unwrap().unabs_ref().is_ok() {
-                    let copy = self.clone();
-                    if SHOW_REDUCTIONS { print!("    {} reduces to ", show_precedence(self, 0, depth)) };
-                    *self = copy.eval().unwrap();
-                    if SHOW_REDUCTIONS { println!("{}", show_precedence(self, 0, depth)) }
+                    self.reduce(depth);
                     true
                 } else {
-                    if !self.lhs_ref_mut().unwrap()._beta_once_normal(depth) {
-                        self.rhs_ref_mut().unwrap()._beta_once_normal(depth)
-                    } else {
-                        true
-                    }
+                    self.lhs_ref_mut().unwrap()._beta_once_normal(depth) ||
+                    self.rhs_ref_mut().unwrap()._beta_once_normal(depth)
                 }
             }
         }
     }
-    
+
     // the return value indicates if reduction was performed
-    fn _beta_once_applicative(&mut self, depth: u32) -> bool {
+    fn _beta_once_applicative_l(&mut self, depth: u32) -> bool {
         match *self {
             Var(_) => false,
-            Abs(_) => self.unabs_ref_mut().unwrap()._beta_once_applicative(depth + 1),
+            Abs(_) => self.unabs_ref_mut().unwrap()._beta_once_applicative_l(depth + 1),
             App(_, _) => {
                 if !self.lhs_ref().unwrap().is_beta_reducible() &&
                    !self.rhs_ref().unwrap().is_beta_reducible() &&
                     self.lhs_ref().unwrap().unabs_ref().is_ok()
                 {
-                    let copy = self.clone();
-                    if SHOW_REDUCTIONS { print!("    {} reduces to ", show_precedence(self, 0, depth)) };
-                    *self = copy.eval().unwrap();
-                    if SHOW_REDUCTIONS { println!("{}", show_precedence(self, 0, depth)) }
+                    self.reduce(depth);
                     true
                 } else {
-                    if !self.lhs_ref_mut().unwrap()._beta_once_applicative(depth) {
-                        self.rhs_ref_mut().unwrap()._beta_once_applicative(depth)
-                    } else {
-                        true
-                    }
+                    self.lhs_ref_mut().unwrap()._beta_once_applicative_l(depth) ||
+                    self.rhs_ref_mut().unwrap()._beta_once_applicative_l(depth)
                 }
             }
         }
     }
-    
+
+    // the return value indicates if reduction was performed
+    fn _beta_once_applicative_r(&mut self, depth: u32) -> bool {
+        match *self {
+            Var(_) => false,
+            Abs(_) => self.unabs_ref_mut().unwrap()._beta_once_applicative_r(depth + 1),
+            App(_, _) => {
+                if !self.rhs_ref().unwrap().is_beta_reducible() &&
+                   !self.lhs_ref().unwrap().is_beta_reducible() &&
+                    self.lhs_ref().unwrap().unabs_ref().is_ok()
+                {
+                    self.reduce(depth);
+                    true
+                } else {
+                    self.rhs_ref_mut().unwrap()._beta_once_applicative_r(depth) ||
+                    self.lhs_ref_mut().unwrap()._beta_once_applicative_r(depth)
+                }
+            }
+        }
+    }
+
     /// Checks whether `self` is β-reducible.
     ///
     /// # Example
@@ -215,14 +239,10 @@ impl Term {
     /// assert_eq!(pred_one, 0.into());
     /// ```
     pub fn beta_full(&mut self) {
-        let mut tmp;
         loop {
             if SHOW_REDUCTIONS { println!("reducing {}", self) }
-            tmp = self.clone();
-            self.beta_once();
-            if *self == tmp { break }
+            if !self.beta_once_indicative() { break }
         }
-
         if SHOW_REDUCTIONS { println!("    doesn't reduce") }
     }
 }
@@ -265,17 +285,30 @@ pub fn beta_once(mut term: Term) -> Term {
 mod test {
     use super::*;
     use parser::parse;
-    
+
     #[test]
     fn evaluation_order() {
         match EVALUATION_ORDER {
             Normal => {
+                let reduces_instantly = parse(&"(λλ1)((λλλ((32)1))(λλ2))").unwrap();
+                assert_eq!(beta_full(reduces_instantly.clone()), beta_once(reduces_instantly));
+
                 let should_reduce = parse(&"(λ2)((λ111)(λ111))").unwrap();
                 assert_eq!(beta_full(should_reduce), Var(1))
             },
-            Applicative => {
-                let shouldnt_reduce = parse(&"(λ2)((λ111)(λ111))").unwrap();
-                assert_eq!(beta_once(shouldnt_reduce), parse(&"(λ2)((λ111)(λ111)(λ111))").unwrap())
+            ApplicativeLeft => {
+                let expr = parse(&"λ1(((λλλ1)1)((λλ21)1))").unwrap();
+                assert_eq!(&format!("{}", beta_once(expr)), "λ1((λλ1)((λλ21)1))");
+
+                let expands = parse(&"(λ2)((λ111)(λ111))").unwrap();
+                assert_eq!(&format!("{}", beta_once(expands)), "(λ2)((λ111)(λ111)(λ111))")
+            },
+            ApplicativeRight => {
+                let expr = parse(&"λ1(((λλλ1)1)((λλ21)1))").unwrap();
+                assert_eq!(&format!("{}", beta_once(expr)), "λ1((λλλ1)1(λ21))");
+
+                let expands = parse(&"(λ2)((λ111)(λ111))").unwrap();
+                assert_eq!(&format!("{}", beta_once(expands)), "(λ2)((λ111)(λ111)(λ111))")
             }
         }
     }
