@@ -17,13 +17,16 @@ pub const DISPLAY_CLASSIC: bool = false;
 /// an applicaction of one term to another.
 #[derive(Debug, PartialEq, Clone)]
 pub enum Term {
+    /// a variable
     Var(usize),
+    /// an abstraction
     Abs(Box<Term>),
+    /// an application
     App(Box<Term>, Box<Term>)
 }
 
 /// An error that can be returned when an inapplicable function is applied to a term.
-#[derive(Debug, PartialEq, Clone)]
+#[derive(Debug, PartialEq)]
 pub enum Error {
     NotAVar,
     NotAnAbs,
@@ -275,7 +278,11 @@ impl fmt::Display for Term {
 pub fn show_precedence(term: &Term, context_precedence: usize, depth: u32) -> String {
     match *term {
         Var(i) => if DISPLAY_CLASSIC {
-            format!("{}", from_u32(depth + 97 - i as u32).unwrap())
+            if depth >= i as u32 {
+                format!("{}", from_u32(depth + 97 - i as u32).unwrap())
+            } else {
+                format!("{}", from_u32(depth + 96 + i as u32).unwrap())
+            }
         } else {
             format!("{:X}", i)
         },
